<?php

/**
 * ApiPublic
 *
 * This class handles the AJAX exhanges with the realestate API from Hello Immo Sync platform
 */

namespace WPIS;

include_once $_SERVER['DOCUMENT_ROOT'] . '/wp-config.php';
include_once $_SERVER['DOCUMENT_ROOT'] . '/wp-load.php';
include_once $_SERVER['DOCUMENT_ROOT'] . '/wp-includes/wp-db.php';
include_once $_SERVER['DOCUMENT_ROOT'] . '/wp-includes/pluggable.php';

class ApiPublic {

	private $apiUrl = 'https://api.immosync.com/realestate/';

	private $headers = [];
	private $payload = [];
	private $errors = [];

	static public function init() {

		$class = new ApiPublic();

		$class->wp_db = $GLOBALS['wpdb'];
		$class->wp_table = $class->wp_db->prefix . 'wpis_connexion';

		$class->headers[] = 'Cache-Control: no-cache';
		$class->headers[] = 'Accept: application/json';
		$class->headers[] = 'Content-Type: application/json';

		$class
			->listenActions();

		$prefixPublic = 'wp_ajax_nopriv_wpis_';
		$prefixPrivate = 'wp_ajax_wpis_';

		add_action( $prefixPublic . 'createContact', [$class, 'createContact'] );
		add_action( $prefixPrivate . 'createContact', [$class, 'createContact'] );

		return $class;
	}

	public function createContact(){

		$payload = json_decode(file_get_contents('php://input'));
		$acceptedSource = ['searchForm', 'contactForm', 'estateForm'];

		$this->log( '[NEW CONTACT]' );
		$this->log( '> rawInput: ' . json_encode($payload) );

		if( !isset($payload->source) || !in_array($payload->source, $acceptedSource) ){
			$this->log( '> error: ' . 'Contact source is not valid' );
			return $this->response(422, ['success' => false, 'data' => 'Contact source is not valid' ]);
		}

		$this->log( '> source: ' . $payload->source );

		$searchZips = [];
		foreach( [1, 2, 3] as $zipKey ){
			if( isset($payload->{'searchZipCode' . $zipKey}) && $payload->{'searchZipCode' . $zipKey}){
				$searchZips[] = $payload->{'searchZipCode' . $zipKey};
			}
		}

		$input = [
			'source'    => $payload->source,
			'phone'     => $payload->phone,
			'firstname' => $payload->firstname,
			'lastname'  => $payload->lastname,
			'email'     => $payload->email,

			'agreement' => isset($payload->agreement) && $payload->agreement,

			'addressNumber' =>  $payload->addressNumber,
			'addressZip'    =>  $payload->addressZip,
			'addressStreet' =>  $payload->addressStreet,
			'addressCity'   =>  $payload->addressCity,
			'addressBox'    =>  $payload->addressBox,

			'message' => $payload->comment
		];

		if( $payload->officeId ){
			$input['officeId'] = $payload->officeId;
		}

		if( $payload->source === 'searchForm' ){
			$input['search'] = [
				'purpose'    => $payload->searchPurpose,
				'category'   => $payload->searchCategory,
				'priceRange' => [$payload->searchPriceMin, $payload->searchPriceMax],
				'rooms'      => $payload->searchRooms,
				'zips'       => $searchZips
			];
		}

		if( $payload->source === 'estateForm' ){
			$input['estateInterestId'] = $payload->estateId;
			$input['message'] = 'ID: ' . $payload->estateId . ' | MESSAGE: '. $payload->comment;
		}

		$this->log( '> formatedInput: ' . json_encode($input) );

		$response = $this->request('POST', 'contact', $input, $this->headers);

		$this->log( '> response : ' . json_encode($response) . "\n" );

		return $this->response(200, ['success' => true, 'data' => json_decode($response)]);
	}

	private function search(){

		$wpisAcccess = $this->wp_db->get_results("SELECT * FROM $this->wp_table LIMIT 1");

		if( empty($wpisAcccess) ){
			$this->errors = 'Wordpress is not connected to WP Immo Connect platform.';
			return $this;
		}

		$protocol = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || $_SERVER['SERVER_PORT'] == 443) ? "https://" : "http://";

		$this->headers[] = 'Access-Control-Allow-Origin: ' . $protocol . $_SERVER['HTTP_HOST'];
		$this->headers[] = 'Authorization: Bearer ' . $wpisAcccess[0]->token;
		$this->headers[] = 'Api-Access-Key:' . $wpisAcccess[0]->access_id;

		return $this;
	}

	private function listenActions() {

		if( !empty($this->errors) ){
			return $this->response(401, ['message' => $this->errors]);
		}

		$this->payload = json_decode(file_get_contents('php://input'));
		$this->payload = $this->payload ? $this->payload : [];
		if( ! isset($_GET['action']) ){
			return false;
		}
		switch( $_GET['action'] ) {
			case 'getRegions':
				$response = $this->request('GET', 'regions');
			break;
			case 'getCategories':
				$response = $this->request('GET', 'categories');
				break;
			case 'getPurposes':
				$response = $this->request('GET', 'purposes');
				break;
			case 'getEstate':
				$response = $this->request('GET', 'estate/' . $_GET['estateId']);
				break;
			case 'getEstates':
				unset($_GET['action']);
				$response = $this->request('GET', 'estates', $_GET);
				break;
			default:
				return;
		}

		if( $response->code !== 200 ){
			return $this->response($response->code, json_decode($response->data));
		}

		$this->response(200, json_decode($response->data));
	}

	public function request(string $method, string $endpoint, array $payload = []){

		$this->search();

		$method = strtoupper($method);

		$params = http_build_query($payload);
		$curl = curl_init();

		$options = [
			CURLOPT_URL => $this->apiUrl . $endpoint . '?' . $params,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_MAXREDIRS => 10,
			CURLOPT_TIMEOUT => 30,
			CURLOPT_CUSTOMREQUEST => $method,
			CURLOPT_HTTPHEADER => $this->headers,
		];

		if( $method === 'POST' ){
			$options[CURLOPT_POST] = true;
			$options[CURLOPT_POSTFIELDS] = http_build_query($payload);
		}

		curl_setopt_array($curl, $options);

		$response = curl_exec($curl);
		$err = curl_error($curl);
		$httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

		curl_close($curl);

		if ($err)
			return (object) ['error' => true, 'message' => "cURL Error : " . $err];

		return (object) ['code' => $httpcode, 'data' => $response];
	}

	private function response( $code, $data = false ) {
		http_response_code($code);
		header('Content-Type: application/json');
		if( $data ){
			echo json_encode($data);
		}
		exit;
	}

	private function log( $message ){
		$cacheDirectory = wp_get_upload_dir()['basedir'] . '/wp-immo-sync';
        $logFile = $cacheDirectory . '/wpis-contact.log';
        $date = (new \DateTime('NOW'))->format("y:m:d H:i:s");
        $logLine = "\r\n" . "{$date} " . $message;
        file_put_contents($logFile, $logLine, FILE_APPEND);
    }
}